# Generated by roxygen2: do not edit by hand

export(RSR)
export(cauchy)
export(extract_log_lik)
export(gamma)
export(gof)
export(laplace)
export(logistic)
export(normal)
export(plot_spatial)
export(projected)
export(stan_colext)
export(stan_distsamp)
export(stan_multinomPois)
export(stan_occu)
export(stan_occuRN)
export(stan_occuTTD)
export(stan_pcount)
export(student_t)
export(turnover)
export(uniform)
exportMethods("$")
exportMethods("[")
exportMethods("[[")
exportMethods(coef)
exportMethods(extract)
exportMethods(fitList)
exportMethods(fitted)
exportMethods(getY)
exportMethods(get_elapsed_time)
exportMethods(get_stancode)
exportMethods(kfold)
exportMethods(loo)
exportMethods(modSel)
exportMethods(names)
exportMethods(nsamples)
exportMethods(plot)
exportMethods(plot_effects)
exportMethods(plot_marginal)
exportMethods(plot_posteriors)
exportMethods(plot_residuals)
exportMethods(posterior_linpred)
exportMethods(posterior_predict)
exportMethods(predict)
exportMethods(ranef)
exportMethods(residuals)
exportMethods(summary)
exportMethods(traceplot)
exportMethods(waic)
import(Rcpp)
importClassesFrom(rstan,stanfit)
importFrom(RSpectra,eigs)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_density)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,unit)
importFrom(grDevices,terrain.colors)
importFrom(graphics,hist)
importFrom(graphics,plot)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(gridExtra,grid.arrange)
importFrom(loo,kfold)
importFrom(loo,loo)
importFrom(loo,loo_compare)
importFrom(loo,waic)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(pbapply,pblapply)
importFrom(pbapply,pboptions)
importFrom(rlang,.data)
importFrom(rstan,extract)
importFrom(rstan,get_elapsed_time)
importFrom(rstan,get_stancode)
importFrom(rstan,sampling)
importFrom(rstan,traceplot)
importFrom(rstantools,nsamples)
importFrom(rstantools,posterior_linpred)
importFrom(rstantools,posterior_predict)
importFrom(stats,as.formula)
importFrom(stats,dbinom)
importFrom(stats,fitted)
importFrom(stats,model.frame)
importFrom(stats,rbinom)
importFrom(stats,residuals)
importFrom(unmarked,"obsCovs<-")
importFrom(unmarked,"siteCovs<-")
importFrom(unmarked,"yearlySiteCovs<-")
importFrom(unmarked,coef)
importFrom(unmarked,getP)
importFrom(unmarked,getY)
importFrom(unmarked,modSel)
importFrom(unmarked,obsCovs)
importFrom(unmarked,predict)
importFrom(unmarked,ranef)
importFrom(unmarked,siteCovs)
importFrom(unmarked,summary)
importFrom(unmarked,yearlySiteCovs)
useDynLib("ubms", .registration = TRUE)
