% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ubms-package.R
\name{ubms}
\alias{ubms}
\alias{ubms-package}
\title{ubms}
\description{
Unmarked Bayesian Models using Stan
}
\author{
Ken Kellner
}
